program conred35;
{$F+}
{
***********************************************************
"Puente de tiristores: carga L-R"
Autores: JRomero - Arodri
Fecha: 10-02-98

		 La simulacin corresponde a un puente de tiristores
		 con carga L-R, y se obeservarn los siguientes
		 parmetros:

		 * tensines de entrada (todas las de  linea), contrastada con la de salida
		  tomando como carga una fuente de corriente.
		 * corriente en la carga y corriente en la fase R
		 * corrientes de cada tiristor.

		 Se podr afectar el ngulo de disparo y
		 el valor del mismo estar acotado entre 0 y 170 grados. El
		 recorrido se har a pasos de 2 grados.

		 El valor del inductor de carga variar de 1mh a 50 mh.

		 OBS: para implementar el disparo por el control arcocoseno
		 se toma como referencia a la fase R y se supone que esta es
		 Vp sin(w*t).
**********************************************************
}

uses
wincrt, xmatdefs, FilUSR, UCntrl, Simul4, traxp, variable,
lib_basi, libep02, libep, libdig01, memorias,dibus;

const w=PI*100;           {frecuencia de red=50 hz}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}
		periodo=20e-3;      {periodo=2*PI/w}
		grillay=10;          {divisiones de la grilla en y}
		grillax=12;			  {divisiones de la grilla en x}
		nper=1;				  {nmero de periodos a visualizar   (nper>=1)}
		rshunt=0.001;		  {valor de resitencias de medicin de corriente}
		NumeroPuntos=512;		{puntos de muestreo}

var   alpha,wt:NReal;
		lcarga:NReal;			{inductacia en la carga}
		{ variables definidas para muestrear UD}
		UD,
		tMuestra,
		NMuestra:Nreal;
		mem1:Tmemoria;

(*********************************************************************
  Funcin para el disparo del tiristor.
*********************************************************************)
{
	El parmetro ALPHA es tomado como control del disparo del tiristor.
}

function disparo(k:integer): boolean;
var vref:NREAL;
begin
	vref:=-cos(alpha*sexrad);
	disparo:=(vref <= sin(w*t - 2*PI/3 - (k-1) * PI/3)) and (cos(w*t - 2*PI/3 - (k-1) * PI/3)>=0);
{	DISPARO:=TRUE;}
end;

(***********************************************************************
	Funciones de definicin del circuito

	Cirt1: componentes no booleanos
	Cirt2: componentes booleanos
***********************************************************************)

procedure cirt1;
begin
vg(1,11,1,311);							{generador R de 220 Vrms}
vg(2,11,2,311);							{generador T de 220 Vrms}
vg(3,11,3,311);							{generador S de 220 Vrms}


{resistencias de medicin de corrientes}
r(1,4,rshunt);
r(2,5,rshunt);
r(3,6,rshunt);
r(1,8,rshunt);
r(2,9,rshunt);
r(3,10,rshunt);

r(7,12,56);								{resistencia de carga}

l(12,0,lcarga);						{L carga}

end;

procedure cirt2;
begin
th(4,7,disparo(1));							{Tiristor 1}
th(0,10,disparo(2));							{Tiristor 2}
th(5,7,disparo(3));							{Tiristor 3}
th(0,8,disparo(4));							{Tiristor 4}
th(6,7,disparo(5));							{Tiristor 5}
th(0,9,disparo(6));							{Tiristor 6}
end;


(************************************************************************
	Definicin de la seal del generador VG[1]
************************************************************************)
procedure funcsr(t:NReal);
begin
rf[1]:=sin(w*t);				{R}
rf[2]:=sin(w*t-2*PI/3);    {S}
rf[3]:=sin(w*t-4*PI/3);    {T}
end;




(************************************************************************
	Procedimientos necesarios de Simeep:

	inicio:	inicializacin de simulaciones, def. de pantalla,etc
	pesca: 	captura e impresin en pantalla de variables observadas
	fin:     procedimiento de finalizacin

************************************************************************)

procedure inicio;
begin
	nn:=12;					{nmero de nodos, sin incluir tierra (0)}
	nindep:=3;     		{nmero de funciones definidas en funcsr = numero de generadores}
	tinicial:=0;			{tiempo inicial de la simulacin}
	tfinal:=nper*periodo;	{tiempo final de la simulacin}


	{parmetros de simulacin}
	dtmax0:=5e-6;
	dtmin:=dtmax0/100;
	PasoDivisor:=6;
	TmaxTmin:=1e3;

	{L carga tiene asociada una variable de estado}
	x0[1]:=0;

	{inicializacin de la unidad de grficos}
	traxp.tinicial:=tinicial;
	traxp.tfinal:=tfinal;

	{inicializacin del modo grfico}
	iniciegr;

	{grillas en pantalla}
	gridx:=grillax;
	gridy:=grillay;

	{matriz de visualizacin de 1 por 4}
	subplot(1,3);


	{primera pantalla}
	definaY(0,-750,150);
	titulo('uD(t) (amarillo) y tensiones de linea: +-RS(rojo),+-ST(negro),+-TR(azul)');
	xlabel('30/DIV');
	ylabel('150 V/DIV');
	linea0;
	grid;

	gridy:=15;
	{segunda pantalla}
	definaY(1,0,10);
	titulo('iD(t) (amarillo) y corriente de tiristores');
	xlabel('30 /DIV');
	ylabel(' 6  5  4  3  2 1  iD');
	grid;

	gridy:=grillay;

	{tercera pantalla}
	definaY(2,-12.5,2.5);
	titulo('i_R=iT1-iT4 (azul) y uR como referencia (rojo)');
	xlabel('30 /DIV');
	ylabel('2.5 A/DIV');
	linea0;
	grid;

	{canal 4,5,6 en la misma pantalla que el canal 0}
	superponga(3,0);
	superponga(4,0);
	superponga(5,0);
	superponga(11,0);
	superponga(12,0);
	superponga(13,0);

	{canal 7,8,9,10,11,12,15 en la misma pantalla que el canal 2}
	superponga(6,1);
	superponga(7,1);
	superponga(8,1);
	superponga(9,1);
	superponga(10,1);
	superponga(15,1);

	{tensin de faseR con iR}
	superponga(14,2);

	{colores a mostrar por canal}

	{primera pantalla}
	definacolor(0,amarillo);			{V_salida}
	definacolor(3,rojo);			{R}
	definacolor(4,negro);      {S}
	definacolor(5,azul);       {T}
	definacolor(11,rojo);			{R}
	definacolor(12,negro);      {S}
	definacolor(13,azul);       {T}

	{segunda pantalla}
	definacolor(1,azul);		{iT1}
	definacolor(6,8);			{iT2}
	definacolor(7,8);			{iT3}
	definacolor(8,azul);		{iT4}
	definacolor(9,8);		{iT5}
	definacolor(10,8);		{iT6}
	definacolor(15,amarillo);	  {iD}

	{tercera pantalla}
	definacolor(2,azul);			{iR}
	definacolor(14,rojo);			{uR}

end;

{____________________________________________________________________}

procedure fin;
begin
TermineGr;
halt;
end;

{____________________________________________________________________}

procedure pesca;

var   k,enx:integer;
		grados:NReal;
begin

	traxp.t:=t;					{indicamos a traxp el tiempo de simulacin}

	wt:=w/sexrad*t;

	{*etiquetas de grficas*}
	grados:=(w/sexrad)*t;
	enX:=PUNTOX+21; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}
	{ANOTAR_EVENTO(numeracin, GRAD1,GRAD2,GRAD, ENX, y,y1a,y1b,y2a,y2b,etiqueta)}
	{alpha}ANOTAR_EVENTO(1,30,30+alpha,grados,enx,30,20,140,20,140, CHR(224)+' para T1');
	{ANOTAR_EVENTO(numeracin, GRAD1,GRAD2,GRAD, ENX, y,y1a,y1b,y2a,y2b,etiqueta)}
	{alpha}ANOTAR_EVENTO(2,30,30+alpha,grados,enx,345,338,458,338,458, CHR(224));


	{captura de valores}

	{primera pantalla}
	trazo(3,v[1]-v[2]);			{R}
	trazo(4,v[2]-v[3]);      {S}
	trazo(5,v[3]-v[1]);       {T}
	trazo(11,v[2]-v[1]);			{R}
	trazo(12,v[3]-v[2]);      {S}
	trazo(13,v[1]-v[3]);       {T}
	trazo(0,v[7]+20);			{V_salida}
	trazo(0,v[7]-20);			{V_salida}

	{segunda pantalla}
	trazo(15,x[1]+130);							{iD}
	trazo(1,(v[1]-v[4])*(1/rshunt)+110);		{iT1}
	trazo(6,(v[10]-v[3])*(1/rshunt)+90);			{iT2}
	trazo(7,(v[2]-v[5])*(1/rshunt)+70);			{iT3}
	trazo(8,(v[8]-v[1])*(1/rshunt)+50);		{iT4}
	trazo(9,(v[3]-v[6])*(1/rshunt)+30);		{iT5}
	trazo(10,(v[9]-v[2])*(1/rshunt)+10);		{iT6}

	trazo(15,130);							{iD}
	trazo(1,110);		{iT1}
	trazo(6,90);			{iT2}
	trazo(7,70);			{iT3}
	trazo(8,50);		{iT4}
	trazo(9,30);		{iT5}
	trazo(10,10);		{iT6}


	{tercera pantalla}
	trazo(2,(2*v[1]-v[4]-v[8])*(1/rshunt));			{iR}
	trazo(14,(v[1]-v[11])*0.04);


	{pesca de UD para muestreo}
	if tMuestra>=periodo/6 then
	begin
		tMuestra:=0;
		NMuestra:=NMuestra+1;

		mem1.fft;
		ud:=mem1.ValorMedio+(trunc(ud)/511)*3;
		mem1.retrocesobarrido(periodo/6);
		if (ud<1) then ud:=0;
	end
	else tMuestra:=t-NMuestra*periodo/6;
	mem1.tic(tMuestra,v[7]);

	if t >= tfinal then
	begin
	{llegamos al final de la simulacin}

		{para el muestreo de UD}
		tMuestra:=0;
		NMuestra:=0;

		for k:= 0 to 15 do
		begin
			borrecanal(k,0);
			desligue(k);
			borde;

			if (k<>1) and (k<>6) and (k<>7) and (k<>8) and (k<>9) and (k<>10) and (k<>15) then
			begin
				grid;
				linea0;
			end
			else
			begin
				gridy:=15;
				grid;
				gridy:=grillay;
			end;
		end;

		{nueva simulacin}
		t:= t-tfinal+tinicial;


	end;



end;


(*********************************************************************
 Principal
*********************************************************************)
BEGIN
	quita:=5;			{separaciones de pantallas}

	{para el muestreo de UD}
	UD:=445;	{valor "inicial", antes de hacer el primer calculo}
	NMuestra:=0;
	tMuestra:=0;
	mem1:= TMemoria.Create_init(periodo/6/numeropuntos,numeropuntos,0,0);


	{decimos a simeep donde estan nuestras funciones}
	FilUSR.Cirt1:=Cirt1;
	FilUSR.Cirt2:=Cirt2;
	FilUSR.FuncsR:=FuncsR;
	FilUSR.Inicio:=Inicio;
	FilUSR.Pesca:=Pesca;
	FilUSR.Fin:=Fin;

	{definicin de controles}
	UCntrl.init;

	UCntrl.insertdisplay(new(PCtrlVerTexto, init
	{ rnglon, texto1  ,            texto2,             texto3}
	(	0,	'Puente de tiristores','con carga L-R.','')));


	UCntrl.insert(new(PCtrlBotonVar,Init
	{renglon, nombre,   unidad,decimales, min, max,   def, inc,    var}
	(  8,   ' Disparo',  ''  ,0        ,   0, 170,     30,  2,  alpha)));

	UCntrl.insert(new(PCtrlBotonVar,Init
	{renglon, nombre,   unidad,decimales, min, max,   def, inc,    var}
	(  9,   ' L-carga',  'mH'  ,0        ,   1, 50,    1,  1,  lcarga)));

	UCntrl.insertdisplay(new(PCtrlVer2Var,init
	{ renglon, nombre1, uni1,deci1,var1 ,nombre2, uni2,deci2,var2}
	(       7, '   Ud',  'V',  0  ,ud ,' grados', '',  1  ,wt)));

	paseo;												  {simulacin}

	mem1.Free;

end.
